#!/bin/bash
# Copyright (C) 2007-2009 Timesys Corporation

FILE=$1

# Use these functions to output whether a package is required or optional, along with flavor text
required() {
	missing
	FATAL=1
	echo "REQUIRED: " $1

}

optional() {
	missing
	OPTIONAL=1
	echo "OPTIONAL: " $1
}

# This function reads the workorder and determines whether or not a software package is selected
package_selected() {
	if [[ $FILE && -e $FILE ]];then
		echo `grep "TSWO_SOFTWARE_$1=y" $FILE`
	fi
}

# Helper Functions
system_check_notice() {
	echo "Refer to the requirements documentation here:"
	echo "https://linuxlink.timesys.com/docs/wiki/factory/FactoryHostRequirements"
	echo
}

missing() {
	if [ -z $MISSING ]; then
		MISSING=1
		echo ""
		echo "====================="
	fi
}

# System Checks
#
# NOTE: Make sure to add a function call at the end of this file when adding
# 	new checks

system_check_env() {
  if [ -n "$LD_LIBRARY_PATH" ]; then
    printf "WARNING: a custom LD_LIBRARY_PATH may lead to broken builds.\n"
  fi
  if [ -n "$ARCH" ]; then
    printf "WARNING: setting ARCH in the environment may lead to broken builds.\n"
  fi
  if [ -n "$CROSS_COMPILE" ]; then
    printf "WARNING: setting CROSS_COMPILE in the environment may lead to broken builds.\n"
  fi
  if [ -n "$CROSSCOMPILE" ]; then
    printf "WARNING: setting CROSSCOMPILE in the environment may lead to broken builds.\n"
  fi
}

system_check_free_space() {
  if [ -n "$(which stat)" ]; then
    FREE_SPACE=$(echo $(($(stat --printf "(%a*%s)/(1024*1024*1024)" --file-system . 2>/dev/null))))
    if [ "$FREE_SPACE" -lt 10 ]; then
			printf "WARNING: %dGB is less than the 10GB+ recommendation for free disk space.\n" $FREE_SPACE
    fi
  fi
}

system_check_gcc() {
	if [ -z $(which gcc 2>/dev/null) ]; then
		required "No host C compiler found."
	fi
}

system_check_gplusplus() {
	if [ -z $(which g++ 2>/dev/null) ]; then
		required "No host C++ compiler found."
	fi
}

system_check_gmake() {
	if [ -z "$(make --version 2>/dev/null | head -n 1 | grep 'GNU Make 3.81')" ]; then
		if [ -z "$(gmake --version 2>/dev/null | head -n 1 | grep 'GNU Make 3.81')" ]; then
			required "Unable to find a suitable GNU make. Please install GNU make 3.81."
		fi
	fi
}

system_check_patch() {
	if [ -z $(which patch 2>/dev/null) ]; then
		required "No host patch utility found."
	fi
}

system_check_bison() {
	if [ -z $(which yacc 2>/dev/null || which bison 2>/dev/null) ]; then
		required "No suitable parser generator (bison) found."
	fi
}

system_check_flex() {
	if [ -z $(which lex 2>/dev/null || which flex 2>/dev/null) ]; then
		required "No suitable lexical analyzer (flex) found."
	fi
}

system_check_perl() {
	if [ -z $(which perl 2>/dev/null) ]; then
		required "No perl interpreter found.  Perl is required for building certain utilities and applications."
	fi
}

system_check_perl_xml__parser() {
	$(perl -MXML::Parser -e 'exit(0);' 2>/dev/null) || \
		required "The XML::Parser perl module is required for some software building certain utilities and applications."
}

system_check_automake() {
	if [ -z $(which automake 2>/dev/null) ]; then
		required "Automake not found.  This is required to build certain utilities and applications"
	fi
}

system_check_shell() {
	$(/bin/sh -c 'echo ${TEST/test/TEST}' 2>/dev/null >&2) || \
		required "/bin/sh does not support common bashisms.  Please relink /bin/sh to /bin/bash."
}

system_check_binutils() {
	if [ ! -f /usr/include/bfd.h ]; then
		required "You need to install the binutils development package."
	fi
}

system_check_guile() {
	if [ -z $(which guile 2>/dev/null) ]; then
		if [ $(package_selected "guile") ]; then
			required "guile not found.  This is required to build guile."
		else
			optional "guile not found.  This is required to build guile."
		fi
	fi
}

system_check_wget() {
	if [ -z $(which wget 2>/dev/null) ]; then
		required "wget not found."
	fi
}

system_check_gettext() {
	if [ -z $(which gettext 2>/dev/null) ]; then
		required "gettext not found."
	fi
}

system_check_texinfo() {
	if [ -z $(which makeinfo 2>/dev/null) ]; then
		optional "makeinfo (texinfo) not found.  This is required to build documentation for some packages."
	fi
}

system_check_gtk() {
	if [ -z $(which gdk-pixbuf-csource 2>/dev/null) ]; then
		if [ $(package_selected "gtk") ]; then
			required "gdk-pixbuf-csource (gtk+2) not found.  The Gtk+ development package provides the gdk-pixbuf-csource that is used when cross building Gtk+ and related packages."
		else
			optional "gdk-pixbuf-csource (gtk+2) not found.  The Gtk+ development package provides the gdk-pixbuf-csource that is used when cross building Gtk+ and related packages."
		fi
	fi
}

system_check_sdl() {
	if [ -z $(which sdl-config 2>/dev/null) ]; then
		if [ $(package_selected sdl) ]; then
			required "sdl-config (SDL) not found.  The SDL development package provides m4 files that are required to cross build SDL and related packages"
		else
			optional "sdl-config (SDL) not found.  The SDL development package provides m4 files that are required to cross build SDL and related packages"
		fi
	fi
}

system_check_swig() {
	if [ -z $(which swig 2>/dev/null) ]; then
		optional "swig not found.  This is required to build some packages."
	fi
}

system_check_python() {
	if [ -z $(which python 2>/dev/null) ]; then
		optional "python not found.  This is required to build some packages."
	fi
}

system_check_python_dev() {
	if [ -z $(ls /usr/include/python*/Python.h 2>/dev/null | head -n1) ]; then
		optional "You need to install the python development package in order to build some packages."
	fi
}

# this is not necessary but we are leaving it in for now
system_check_icon_naming_utils() {
	$(pkg-config --exists icon-naming-utils 2>/dev/null) || \
		optional "icon-naming-utils not found.  This is required for certain graphical icon themes."
}

system_check_glib_genmarshal() {
	if [ -z $(which glib-genmarshal 2>/dev/null) ]; then
		optional "glib-genmarshal (glib2 development) not found.  This is required to build some packages."
	fi
}

system_check_dbus_binding_tool() {
	if [ -z $(which dbus-binding-tool 2>/dev/null) ]; then
		optional "dbus-binding-tool (dbus-glib) not found.  This is required to build some packages."
	fi
}

system_check_xsltproc() {
	if [ -z $(which xsltproc 2>/dev/null) ]; then
		optional "xsltproc not found.  This is required to build some graphical packages."
	fi
}

system_check_gawk() {
	if [ -z $(which gawk 2>/dev/null) ]; then
		required "gawk not found.  GNU awk is required for building GNU packages such as glibc."
	fi
}

system_check_zlib() {
	if [ ! -f /usr/include/zlib.h ]; then
		required "You need to install the zlib development package."
	fi
}

system_check_ncurses() {
	if [ ! -f /usr/include/ncurses.h ]; then
		required "You need to install the ncurses development package."
	fi
	if [ ! -f /usr/lib/libncurses.a ]; then
		required "You need to install the ncurses development and/or static package(s)."
	fi
}

system_check_libtool() {
	if [ -z $(which libtoolize 2>/dev/null) ]; then
		optional "libtoolize not found.  libtool is required for building some packages."
	fi
}

system_check_zip() {
	if [ -z $(which zip 2>/dev/null) ]; then
		optional "zip not found.  zip is required for building ltp (Linux Test Project)."
	fi
}

system_check_ecj() {
	if [ -z $(which ecj 2>/dev/null) ]; then
		optional "ecj not found.  ecj is required for building GNU Classpath."
	fi
}

system_check_fastjar() {
	if [ -z $(which fastjar 2>/dev/null) ]; then
		optional "fastjar not found.  fastjar is required for building GNU Classpath."
	fi
}

system_check_libc_a() {
	if [ ! -f /usr/lib/libc.a ]; then
	  if [ ! -f /usr/lib64/libc.a ]; then
	    if [ ! -f /lib/libc.a ]; then
	      if [ ! -f /lib64/libc.a ]; then
		      required "You need to install the development and/or static glibc package(s)."
        fi
      fi
    fi
  fi
}

# Function Calls
system_check_free_space
system_check_env
system_check_gcc
system_check_gplusplus
system_check_gmake
system_check_patch
system_check_bison
system_check_flex
system_check_perl
system_check_perl_xml__parser
system_check_automake
system_check_shell
system_check_binutils
system_check_guile
system_check_wget
system_check_gettext
system_check_texinfo
system_check_gtk
system_check_sdl
system_check_swig
system_check_python
system_check_python_dev
system_check_icon_naming_utils
system_check_glib_genmarshal
system_check_dbus_binding_tool
system_check_xsltproc
system_check_gawk
system_check_ncurses
system_check_zlib
system_check_libtool
system_check_zip
system_check_ecj
system_check_fastjar
system_check_libc_a

if [ $MISSING ]; then
	echo "====================="
	echo ""

	if [[ $OPTIONAL ]]; then
		echo "WARNING: Some optional host tools are missing.  Some target packages may fail to build."
	fi
	if [[ $FATAL ]]; then
		echo "ERROR: Some necessary host tools are missing.  Please install them and run 'make checksystem' again."
	fi

	echo ""
	system_check_notice

	if [[ ! $FATAL ]]; then
		echo "SUCCESS: All necessary host tools are installed.  However, some optional packages are not installed, and may be required to build certain packages."
  else
    exit 1
	fi
else
	echo "SUCCESS: All necessary host tools are installed."
fi
